@@ -1,7 +1,8 @@
 import getpass
-import optparse
+import logging
 import re
-from typing import Dict, List, Optional, Union
+from argparse import Namespace
+from typing import Dict, List, Optional
 
 import attr
 import psutil
@@ -9,18 +10,34 @@ import psycopg2
 import psycopg2.extras
 from psycopg2 import sql
 from psycopg2.errors import (
+    FeatureNotSupported,
     InterfaceError,
     InvalidPassword,
+    InsufficientPrivilege,
     OperationalError,
     ProgrammingError,
+    QueryCanceled,
 )
 from psycopg2.extensions import connection
 
 from . import queries
-from .types import BlockingProcess, Filters, WaitingProcess, RunningProcess, NO_FILTER
+from .types import (
+    BlockingProcess,
+    FailedQueriesInfo,
+    Filters,
+    WaitingProcess,
+    Pct,
+    RunningProcess,
+    ServerInformation,
+    TempFileInfo,
+    NO_FILTER,
+)
 from .utils import clean_str
 
 
+logger = logging.getLogger("pgactivity")
+
+
 def pg_get_version(pg_conn: connection) -> str:
     """Get PostgreSQL server version."""
     query = queries.get("get_version")
@@ -69,6 +86,7 @@ class Data:
     min_duration: float
     filters: Filters
     dsn_parameters: Dict[str, str]
+    failed_queries: FailedQueriesInfo
 
     @classmethod
     def pg_connect(
@@ -102,17 +120,13 @@ class Data:
                 cur.execute(queries.get("disable_log_min_duration_statement"))
                 if pg_conn.server_version >= 130000:
                     cur.execute(queries.get("disable_log_min_duration_sample"))
-            if not rds_mode:  # Make sure we are using superuser if not on RDS
-                cur.execute(queries.get("is_superuser"))
-                ret = cur.fetchone()
-                if ret[0] != "on":
-                    raise Exception("Must be run with database superuser privileges.")
         pg_version = pg_get_short_version(pg_get_version(pg_conn))
         return cls(
             pg_conn,
             pg_version,
             pg_conn.server_version,
             min_duration=min_duration,
+            failed_queries=FailedQueriesInfo(),
             filters=filters,
             dsn_parameters=pg_conn.info.dsn_parameters,
         )
@@ -132,27 +146,44 @@ class Data:
 
     def pg_is_local_access(self) -> bool:
         """
-        Verify if the user running pg_activity can acces
-        system informations for the postmaster process.
+        Verify if the user running pg_activity can access
+        system information for the postmaster process.
         """
-        try:
-            query = queries.get("get_pid_file")
-            with self.pg_conn.cursor() as cur:
+        query = queries.get("get_data_directory")
+        with self.pg_conn.cursor() as cur:
+            try:
                 cur.execute(query)
-                ret = cur.fetchone()
-            pid_file = ret["pid_file"]
+            except InsufficientPrivilege:
+                logger.info(
+                    "Insufficient privilege to show data_directory. System counters are disabled."
+                )
+                return False
+            ret = cur.fetchone()
+
+        pid_file = f"{ret['data_directory']}/postmaster.pid"
+        try:
             with open(pid_file, "r") as fd:
-                pid = fd.readlines()[0].strip()
-                try:
-                    proc = psutil.Process(int(pid))
-                    proc.io_counters()
-                    proc.cpu_times()
-                    return True
-                except psutil.AccessDenied:
-                    return False
-                except Exception:
-                    return False
-        except Exception:
+                pid = fd.readline().strip()
+        except PermissionError:
+            logger.info("Access Denied to the pidfile. System counters are disabled.")
+            return False
+
+        try:
+            proc = psutil.Process(int(pid))
+            proc.io_counters()
+            proc.cpu_times()
+            return True
+        except psutil.AccessDenied:
+            logger.info(
+                "Access denied to the psutil data. System counters are disabled."
+            )
+            return False
+        except AttributeError:
+            # See issue #300
+            logger.info(
+                "Your platform doesn't support some of psutil features required to get system counters. "
+                "System counters are disabled."
+            )
             return False
 
     def pg_cancel_backend(self, pid: int) -> bool:
@@ -178,81 +209,223 @@ class Data:
             ret: Dict[str, bool] = cur.fetchone()
         return ret["is_stopped"]
 
-    DbInfoDict = Dict[str, Union[str, int, float]]
-
-    def pg_get_db_info(
-        self,
-        prev_db_infos: Optional[DbInfoDict],
-        using_rds: bool = False,
-        skip_sizes: bool = False,
-    ) -> DbInfoDict:
+    def pg_get_temporary_file(self) -> Optional[TempFileInfo]:
         """
-        Get current sum of transactions, total size and  timestamp.
+        Count the number of temporary files and get their total size
         """
-        prev_total_size = "0"
-        if prev_db_infos is not None:
-            prev_total_size = prev_db_infos["total_size"]  # type: ignore[assignment]
+        if self.failed_queries.temp_file_query_failed:
+            # prevent a spam of errors in PostgreSQL logs if we already failed once
+            # for lack of privilege or timeout
+            return None
+
+        if self.pg_num_version >= 120000:
+            query = queries.get("get_temporary_files_post_120000")
+        elif self.pg_num_version >= 90100:
+            query = queries.get("get_temporary_files_post_090100")
+        else:
+            query = queries.get("get_temporary_files_oldest")
 
-        query = queries.get("get_db_info")
         with self.pg_conn.cursor() as cur:
-            cur.execute(
-                query,
-                {
-                    "skip_db_size": skip_sizes,
-                    "prev_total_size": prev_total_size,
-                    "using_rds": using_rds,
-                    "dbname_filter": self.filters.dbname,
-                },
-            )
-            ret = cur.fetchone()
-        tps = 0
-        size_ev = 0.0
-        if prev_db_infos is not None:
-            dt = float(ret["timestamp"] - prev_db_infos["timestamp"])
             try:
-                tps = int((ret["no_xact"] - prev_db_infos["no_xact"]) / dt)
-                size_ev = float(ret["total_size"] - prev_db_infos["total_size"]) / dt
+                cur.execute(queries.get("set_statement_timeout"), {"timeout": "400ms"})
+                cur.execute(query)
+                ret = cur.fetchone()
+            except InsufficientPrivilege:
+                # superuser or pg_read_server_files are required (Issue #278)
+                cur.execute(queries.get("reset_statement_timeout"))
+                self.failed_queries.temp_file_query_failed = True
+                logger.info(
+                    "Insufficient privilege to fetch the tempfile data. "
+                    "The feature was disabled. Please use --no-tempfiles or a platform specific setting (eg. --rds)."
+                )
+
+                return None
+            except QueryCanceled:
+                # if an excessive amount of tempfile exists, the query could be very long
+                # to avoid such a case we set a statement_timeout shorter than the lowest
+                # refresh rate. This could end up spamming the PostgreSQL logs.
+                self.failed_queries.temp_file_query_failed = True
+                cur.execute(queries.get("reset_statement_timeout"))
+                logger.info(
+                    "The tempfile query ended in a timeout. "
+                    "The feature was disabled. Check the temporary files on the server."
+                )
+                return None
+            cur.execute(queries.get("reset_statement_timeout"))
+
+        return TempFileInfo(**ret)
+
+    def pg_get_wal_senders(self) -> Optional[int]:
+        """
+        Count the number of wal senders
+        """
+        if self.pg_num_version >= 90100:
+            query = queries.get("get_wal_senders_post_090100")
+        else:
+            return None
+
+        with self.pg_conn.cursor() as cur:
+            cur.execute(query)
+            ret = cur.fetchone()
+
+        return int(ret["wal_senders"])
+
+    def pg_get_wal_receivers(self) -> Optional[int]:
+        """
+        Count the number of wal receivers
+        """
+        if self.failed_queries.wal_receivers_query_failed:
+            # prevent a spam of errors un PostgreSQL logs if we already failed once
+            # because the feature is not implemented
+            return None
+
+        if self.pg_num_version >= 90600:
+            query = queries.get("get_wal_receivers_post_090600")
+        else:
+            return None
+
+        try:
+            with self.pg_conn.cursor() as cur:
+                cur.execute(query)
+                ret = cur.fetchone()
+        except FeatureNotSupported:
+            # Not implemented on Aurora (Issue #301)
+            self.failed_queries.wal_receivers_query_failed = True
+            logger.info(
+                "The receiver information is not available on your platform. "
+                "The feature is disabled. Please use the --no-walreceiver option."
+            )
+            return None
+
+        return int(ret["wal_receivers"])
+
+    def pg_get_replication_slots(self) -> Optional[int]:
+        """
+        Count the number of replication slots
+        """
+        if self.pg_num_version >= 140000:
+            query = queries.get("get_replication_slots_post_140000")
+        else:
+            return None
+
+        with self.pg_conn.cursor() as cur:
+            cur.execute(query)
+            ret = cur.fetchone()
+
+        return int(ret["replication_slots"])
+
+    def pg_get_server_information(
+        self,
+        prev_server_info: Optional[ServerInformation] = None,
+        using_rds: bool = False,
+        skip_db_size: bool = False,
+        skip_tempfile: bool = False,
+        skip_walreceiver: bool = False,
+    ) -> ServerInformation:
+        """
+        Get the server information (session, workers, cache hit ratio etc..)
+        """
+
+        prev_total_size = 0
+        if prev_server_info is not None:
+            prev_total_size = prev_server_info.total_size
+
+        if self.pg_num_version >= 110000:
+            query = queries.get("get_server_info_post_110000")
+        elif self.pg_num_version >= 100000:
+            query = queries.get("get_server_info_post_100000")
+        elif self.pg_num_version >= 90600:
+            query = queries.get("get_server_info_post_090600")
+        elif self.pg_num_version >= 90400:
+            query = queries.get("get_server_info_post_090400")
+        elif self.pg_num_version >= 90200:
+            query = queries.get("get_server_info_post_090200")
+        elif self.pg_num_version >= 90000:
+            query = queries.get("get_server_info_post_090000")
+        else:
+            query = queries.get("get_server_info_oldest")
+
+        with self.pg_conn.cursor() as cur:
+            try:
+                cur.execute(
+                    query,
+                    {
+                        "dbname_filter": self.filters.dbname,
+                        "skip_db_size": skip_db_size,
+                        "prev_total_size": prev_total_size,
+                        "using_rds": using_rds,
+                    },
+                )
+            except InsufficientPrivilege:
+                logger.info(
+                    "Privileges might be insufficient to connect to a database. "
+                    "Try to use a --filter, the --no-db-size option or a platform specific setting (eg. --rds)"
+                )
+                raise
+            ret = cur.fetchone()
+
+        temporary_file_info: Optional[TempFileInfo] = None
+        if not skip_tempfile:
+            temporary_file_info = self.pg_get_temporary_file()
+        wal_senders = self.pg_get_wal_senders()
+        wal_receivers: Optional[int] = None
+        if not skip_walreceiver:
+            wal_receivers = self.pg_get_wal_receivers()
+        replication_slots = self.pg_get_replication_slots()
+
+        hr: Optional[Pct] = None
+        tps, ips, ups, dps, rps = 0, 0, 0, 0, 0
+        size_ev = 0.0
+        if prev_server_info is not None:
+            dt = float(ret["epoch"] - prev_server_info.epoch)
+            try:
+                # Note: All this could be negative just after a stat reset
+                tps = int((ret["xact_count"] - prev_server_info.xact_count) / dt)
+                size_ev = float(ret["total_size"] - prev_server_info.total_size) / dt
+                ips = int((ret["insert"] - prev_server_info.insert) / dt)
+                ups = int((ret["update"] - prev_server_info.update) / dt)
+                dps = int((ret["delete"] - prev_server_info.delete) / dt)
+                rps = int(
+                    (ret["tuples_returned"] - prev_server_info.tuples_returned) / dt
+                )
+                deltaread = ret["blks_read"] - prev_server_info.blks_read
+                deltahit = ret["blks_hit"] - prev_server_info.blks_hit
+                if deltaread + deltahit != 0:
+                    hr = Pct(100 * deltahit / (deltaread + deltahit))
             except ZeroDivisionError:
                 pass
-        return {
-            "timestamp": ret["timestamp"],
-            "no_xact": ret["no_xact"],
-            "total_size": ret["total_size"],
-            "max_length": ret["max_length"],
-            "tps": tps,
-            "size_ev": size_ev,
-        }
 
-    def pg_get_active_connections(self) -> int:
-        """
-        Get total of active connections.
-        """
-
-        if self.pg_num_version < 90200:
-            query = queries.get("get_active_connections")
-        else:
-            query = queries.get("get_active_connections_post_90200")
-
-        with self.pg_conn.cursor() as cur:
-            cur.execute(query, {"dbname_filter": self.filters.dbname})
-            ret = cur.fetchone()
-        active_connections = int(ret["active_connections"])
-        return active_connections
+        return ServerInformation(
+            size_evolution=size_ev,
+            tps=tps,
+            insert_per_second=ips,
+            update_per_second=ups,
+            delete_per_second=dps,
+            tuples_returned_per_second=rps,
+            cache_hit_ratio_last_snap=hr,
+            temporary_file=temporary_file_info,
+            wal_senders=wal_senders,
+            wal_receivers=wal_receivers,
+            replication_slots=replication_slots,
+            **ret,
+        )
 
     def pg_get_activities(self, duration_mode: int = 1) -> List[RunningProcess]:
         """
         Get activity from pg_stat_activity view.
         """
-        if self.pg_num_version >= 110000:
+        if self.pg_num_version >= 130000:
+            qs = queries.get("get_pg_activity_post_130000")
+        elif self.pg_num_version >= 110000:
             qs = queries.get("get_pg_activity_post_110000")
         elif self.pg_num_version >= 100000:
             qs = queries.get("get_pg_activity_post_100000")
         elif self.pg_num_version >= 90600:
-            qs = queries.get("get_pg_activity_post_90600")
+            qs = queries.get("get_pg_activity_post_090600")
         elif self.pg_num_version >= 90200:
-            qs = queries.get("get_pg_activity_post_90200")
+            qs = queries.get("get_pg_activity_post_090200")
         else:
-            qs = queries.get("get_pg_activity")
+            qs = queries.get("get_pg_activity_oldest")
 
         duration_column = self.get_duration_column(duration_mode)
         query = sql.SQL(qs).format(duration_column=sql.Identifier(duration_column))
@@ -274,9 +447,9 @@ class Data:
         Get waiting queries.
         """
         if self.pg_num_version >= 90200:
-            qs = queries.get("get_waiting_post_90200")
+            qs = queries.get("get_waiting_post_090200")
         else:
-            qs = queries.get("get_waiting")
+            qs = queries.get("get_waiting_oldest")
 
         duration_column = self.get_duration_column(duration_mode)
         query = sql.SQL(qs).format(duration_column=sql.Identifier(duration_column))
@@ -297,11 +470,11 @@ class Data:
         Get blocking queries
         """
         if self.pg_num_version >= 90600:
-            qs = queries.get("get_blocking_post_90600")
+            qs = queries.get("get_blocking_post_090600")
         elif self.pg_num_version >= 90200:
-            qs = queries.get("get_blocking_post_90200")
+            qs = queries.get("get_blocking_post_090200")
         else:
-            qs = queries.get("get_blocking")
+            qs = queries.get("get_blocking_oldest")
 
         duration_column = self.get_duration_column(duration_mode)
         query = sql.SQL(qs).format(duration_column=sql.Identifier(duration_column))
@@ -348,8 +521,7 @@ class Data:
 
 
 def pg_connect(
-    options: optparse.Values,
-    dsn: str,
+    options: Namespace,
     exit_on_failed: bool = True,
     min_duration: float = 0.0,
     filters: Filters = NO_FILTER,
@@ -359,7 +531,7 @@ def pg_connect(
     for nb_try in range(2):
         try:
             data = Data.pg_connect(
-                dsn=dsn,
+                dsn=options.connection_string,
                 host=options.host,
                 port=options.port,
                 user=options.username,
