@@ -1,5 +1,6 @@
 import enum
 import functools
+from datetime import timedelta
 from typing import (
     Any,
     Callable,
@@ -24,9 +25,11 @@ from attr import validators
 from . import compat, colors, utils
 
 
+class Pct(float):
+    """Used to distinguish percentage from float when displaying the header"""
+
+
 T = TypeVar("T")
-
-
 E = TypeVar("E", bound=enum.IntEnum)
 
 
@@ -168,17 +171,6 @@ class SortKey(enum.Enum):
         return cls.duration
 
 
-@enum.unique
-class QueryDisplayMode(enum.IntEnum):
-    truncate = 1
-    wrap_noindent = 2
-    wrap = 3
-
-    @classmethod
-    def default(cls) -> "QueryDisplayMode":
-        return cls.wrap_noindent
-
-
 @enum.unique
 class QueryMode(enum.Enum):
     activities = "running queries"
@@ -238,7 +230,9 @@ class Column:
     justify: str = attr.ib(
         "left", validator=validators.in_(["left", "center", "right"])
     )
-    transform: Callable[[Any], str] = attr.ib(default=str, repr=False)
+    transform: Callable[[Any], str] = attr.ib(
+        default=lambda v: str(v) if v is not None else "", repr=False
+    )
     color_key: Union[str, Callable[[Any], str]] = attr.ib(
         default=_color_key_marker, repr=False
     )
@@ -292,14 +286,15 @@ class UI:
     duration_mode: DurationMode = attr.ib(
         default=DurationMode.query, converter=DurationMode
     )
-    query_display_mode: QueryDisplayMode = attr.ib(
-        default=QueryDisplayMode.default(), converter=QueryDisplayMode
-    )
+    wrap_query: bool = False
     sort_key: SortKey = attr.ib(default=SortKey.default(), converter=SortKey)
     query_mode: QueryMode = attr.ib(default=QueryMode.activities, converter=QueryMode)
     refresh_time: Union[float, int] = 2
     in_pause: bool = False
     interactive_timeout: Optional[int] = None
+    show_instance_info_in_header: bool = True
+    show_system_info_in_header: bool = True
+    show_worker_info_in_header: bool = True
 
     @classmethod
     def make(
@@ -579,6 +574,51 @@ class UI:
         """
         self.in_pause = not self.in_pause
 
+    def toggle_system_info_in_header(self) -> None:
+        """Toggle the 'show_system_info_in_header' attribute.
+
+        >>> ui = UI.make()
+        >>> ui.show_system_info_in_header
+        True
+        >>> ui.toggle_system_info_in_header()
+        >>> ui.show_system_info_in_header
+        False
+        >>> ui.toggle_system_info_in_header()
+        >>> ui.show_system_info_in_header
+        True
+        """
+        self.show_system_info_in_header = not self.show_system_info_in_header
+
+    def toggle_instance_info_in_header(self) -> None:
+        """Toggle the 'show_instance_info_in_header' attribute.
+
+        >>> ui = UI.make()
+        >>> ui.show_instance_info_in_header
+        True
+        >>> ui.toggle_instance_info_in_header()
+        >>> ui.show_instance_info_in_header
+        False
+        >>> ui.toggle_instance_info_in_header()
+        >>> ui.show_instance_info_in_header
+        True
+        """
+        self.show_instance_info_in_header = not self.show_instance_info_in_header
+
+    def toggle_worker_info_in_header(self) -> None:
+        """Toggle the 'show_worker_info_in_header' attribute.
+
+        >>> ui = UI.make()
+        >>> ui.show_worker_info_in_header
+        True
+        >>> ui.toggle_worker_info_in_header()
+        >>> ui.show_worker_info_in_header
+        False
+        >>> ui.toggle_worker_info_in_header()
+        >>> ui.show_worker_info_in_header
+        True
+        """
+        self.show_worker_info_in_header = not self.show_worker_info_in_header
+
     def evolve(self, **changes: Any) -> None:
         """Return a new UI with 'changes' applied.
 
@@ -595,7 +635,7 @@ class UI:
             return
         forbidden = set(changes) - {
             "duration_mode",
-            "query_display_mode",
+            "wrap_query",
             "sort_key",
             "query_mode",
             "refresh_time",
@@ -645,21 +685,59 @@ class Host:
     dbname: str
 
 
-@attr.s(auto_attribs=True, slots=True)
-class DBInfo:
-    total_size: int
-    size_ev: int
-
-
-@attr.s(auto_attribs=True, slots=True)
-class MemoryInfo:
-    percent: float
+@attr.s(auto_attribs=True, slots=True, frozen=True)
+class SwapInfo:
     used: int
+    free: int
+    total: int
+
+    @classmethod
+    def default(cls) -> "SwapInfo":
+        return cls(0, 0, 0)
+
+    @property
+    def pct_used(self) -> Optional[Pct]:
+        if self.total == 0:
+            # account for the zero swap case (#318)
+            return None
+        return Pct(self.used * 100 / self.total)
+
+    @property
+    def pct_free(self) -> Optional[Pct]:
+        if self.total == 0:
+            # account for the zero swap case (#318)
+            return None
+        return Pct(self.free * 100 / self.total)
+
+
+@attr.s(auto_attribs=True, slots=True, frozen=True)
+class MemoryInfo:
+    used: int
+    buff_cached: int
+    free: int
     total: int
 
     @classmethod
     def default(cls) -> "MemoryInfo":
-        return cls(0.0, 0, 0)
+        return cls(0, 0, 0, 0)
+
+    @property
+    def pct_used(self) -> Optional[Pct]:
+        if self.total == 0:
+            return None
+        return Pct(self.used * 100 / self.total)
+
+    @property
+    def pct_free(self) -> Optional[Pct]:
+        if self.total == 0:
+            return None
+        return Pct(self.free * 100 / self.total)
+
+    @property
+    def pct_bc(self) -> Optional[Pct]:
+        if self.total == 0:
+            return None
+        return Pct(self.buff_cached * 100 / self.total)
 
 
 @attr.s(auto_attribs=True, slots=True)
@@ -687,7 +765,7 @@ class IOCounter:
 @attr.s(auto_attribs=True, frozen=True, slots=True)
 class SystemInfo:
     memory: MemoryInfo
-    swap: MemoryInfo
+    swap: SwapInfo
     load: LoadAverage
     io_read: IOCounter
     io_write: IOCounter
@@ -698,28 +776,97 @@ class SystemInfo:
         cls,
         *,
         memory: Optional[MemoryInfo] = None,
-        swap: Optional[MemoryInfo] = None,
+        swap: Optional[SwapInfo] = None,
         load: Optional[LoadAverage] = None,
     ) -> "SystemInfo":
         """Zero-value builder.
 
         >>> SystemInfo.default()  # doctest: +NORMALIZE_WHITESPACE
-        SystemInfo(memory=MemoryInfo(percent=0.0, used=0, total=0),
-                   swap=MemoryInfo(percent=0.0, used=0, total=0),
+        SystemInfo(memory=MemoryInfo(used=0, buff_cached=0, free=0, total=0),
+                   swap=SwapInfo(used=0, free=0, total=0),
                    load=LoadAverage(avg1=0.0, avg5=0.0, avg15=0.0),
                    io_read=IOCounter(count=0, bytes=0, chars=0),
-                   io_write=IOCounter(count=0, bytes=0, chars=0),
-                   max_iops=0)
+                   io_write=IOCounter(count=0, bytes=0, chars=0), max_iops=0)
         """
         return cls(
             memory or MemoryInfo.default(),
-            swap or MemoryInfo.default(),
+            swap or SwapInfo.default(),
             load or LoadAverage.default(),
             IOCounter.default(),
             IOCounter.default(),
+            0,
         )
 
 
+@attr.s(frozen=False, auto_attribs=True, slots=True)
+class FailedQueriesInfo:
+    temp_file_query_failed: bool = False
+    wal_receivers_query_failed: bool = False
+
+
+@attr.s(frozen=True, auto_attribs=True, slots=True)
+class TempFileInfo:
+    temp_files: int
+    temp_bytes: int
+
+
+@attr.s(frozen=True, auto_attribs=True, slots=True)
+class ServerInformation:
+    # Fetched from the database
+    xact_count: int
+    insert: int
+    update: int
+    delete: int
+    tuples_returned: int
+    total_size: int
+    blks_read: int
+    blks_hit: int
+    max_dbname_length: int
+    uptime: timedelta
+    epoch: int  # an epoch,  used for the calculation of the tps & size_evolution
+    active_connections: int
+    idle: int
+    idle_in_transaction: int
+    idle_in_transaction_aborted: int
+    total: int
+    waiting: int
+    max_connections: int
+    autovacuum_workers: Optional[int]
+    autovacuum_max_workers: int
+    logical_replication_workers: Optional[int]
+    parallel_workers: Optional[int]
+    max_logical_replication_workers: Optional[int]
+    max_parallel_workers: Optional[int]
+    max_worker_processes: Optional[int]
+    max_wal_senders: Optional[int]
+    max_replication_slots: Optional[int]
+    wal_senders: Optional[int]
+    wal_receivers: Optional[int]
+    replication_slots: Optional[int]
+    temporary_file: Optional[TempFileInfo]
+    # Computed in data.pg_get_server_information()
+    size_evolution: float
+    tps: int
+    insert_per_second: int
+    update_per_second: int
+    delete_per_second: int
+    tuples_returned_per_second: int
+    cache_hit_ratio_last_snap: Optional[Pct] = attr.ib(
+        converter=attr.converters.optional(Pct)
+    )
+
+    @property
+    def worker_processes(self) -> Optional[int]:
+        if self.parallel_workers is None and self.logical_replication_workers is None:
+            return None
+        else:
+            return (0 if self.parallel_workers is None else self.parallel_workers) + (
+                0
+                if self.logical_replication_workers is None
+                else self.logical_replication_workers
+            )
+
+
 class LockType(enum.Enum):
     """Type of lockable object
 
@@ -758,6 +905,7 @@ class BaseProcess:
     duration: Optional[float]
     state: str
     query: Optional[str]
+    query_leader_pid: Optional[int]
     is_parallel_worker: bool
 
 
@@ -766,6 +914,7 @@ class RunningProcess(BaseProcess):
     """Process for a running query."""
 
     wait: Union[bool, None, str]
+    query_leader_pid: Optional[int]
     is_parallel_worker: bool
 
 
@@ -779,7 +928,8 @@ class WaitingProcess(BaseProcess):
     type: LockType = attr.ib(converter=locktype)
     relation: str
 
-    # TODO: update queries to select/compute this column.
+    # TODO: update queries to select/compute these column.
+    query_leader_pid: Optional[int] = attr.ib(default=None, init=False)
     is_parallel_worker: bool = attr.ib(default=False, init=False)
 
 
@@ -794,7 +944,8 @@ class BlockingProcess(BaseProcess):
     relation: str
     wait: Union[bool, None, str]
 
-    # TODO: update queries to select/compute this column.
+    # TODO: update queries to select/compute these column.
+    query_leader_pid: Optional[int] = attr.ib(default=None, init=False)
     is_parallel_worker: bool = attr.ib(default=False, init=False)
 
 
